@@ -1,8 +1,16 @@
 #ifndef EXTERNAL_STATS_H
 #define EXTERNAL_STATS_H
 
+#include <Rversion.h>
+
+#if R_VERSION >= R_Version(3, 6, 2)
+#define USE_FC_LEN_T
+#endif
+
 #include <Rmath.h> // used to pull in qchisq, et al
 
+#undef USE_FC_LEN_T
+
 #ifdef __cplusplus
 extern "C" {
 #endif
