@@ -4,18 +4,27 @@
 #include <stdbool.h>
 #include <misc/stddef.h>
 
-#include <external/linearAlgebra.h>
-
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 // z = alpha * x + y; z must be distinct from x and y
-extern void (*misc_addVectors)(const double* restrict x, misc_size_t length, double alpha, const double* restrict y, double* restrict z);
+void misc_addVectors(const double* restrict x, misc_size_t length, const double* restrict y, double* restrict z);
+void misc_subtractVectors(const double* restrict x, misc_size_t length, const double* restrict y, double* restrict z);
+void misc_addVectorsWithMultiplier(const double* restrict x, size_t length, double alpha, const double* restrict y, double* restrict z);
+
+// y := alpha * x + y
+extern void (*misc_addVectorsInPlace)(const double* restrict x, misc_size_t length, double* restrict y);
+extern void (*misc_subtractVectorsInPlace)(const double* restrict x, misc_size_t length, double* restrict y);
+extern void (*misc_addVectorsInPlaceWithMultiplier)(const double* restrict x, misc_size_t length, double alpha, double* restrict y);
+extern void (*misc_addAlignedVectorsInPlace)(const double* restrict x, misc_size_t length, double* restrict y);
+extern void (*misc_subtractAlignedVectorsInPlace)(const double* restrict x, misc_size_t length, double* restrict y);
+
+// x := x + alpha
+extern void (*misc_addScalarToVectorInPlace)(double* x, misc_size_t length, double alpha);
  
 // x: = alpha
 bool misc_vectorIsConstant(const double* d, misc_size_t length);
-// void misc_setVectorToConstant(double* x, misc_size_t length, double alpha);
 extern void (*misc_setVectorToConstant)(double* x, misc_size_t length, double alpha);
 void misc_setIndexedVectorToConstant(double* restrict x, const misc_size_t* restrict indices, misc_size_t length, double alpha);
   
@@ -34,6 +43,8 @@ void misc_hadamardMultiplyVectorsInPlace(double* restrict x, misc_size_t length,
 double misc_sumVectorElements(const double* x, misc_size_t length);
 double misc_sumIndexedVectorElements(const double* x, const misc_size_t* indices, misc_size_t length);
 
+extern void (*misc_transposeMatrix)(const double* restrict x, misc_size_t numRows, misc_size_t numCols, double* restrict xt);
+
 void misc_multiplyMatrixIntoVector(const double* restrict matrix, misc_size_t numRows, misc_size_t numCols, int useTranspose,
                                    const double* restrict vector, double* restrict result);
 
@@ -41,10 +52,5 @@ void misc_multiplyMatrixIntoVector(const double* restrict matrix, misc_size_t nu
 }
 #endif
 
-#define misc_addVectorsInPlace ext_addVectorsInPlace
-#define misc_addScalarToVectorInPlace ext_addScalarToVectorInPlace
-#define misc_sumSquaresOfVectorElements ext_sumSquaresOfVectorElements
-#define misc_transposeMatrix ext_transposeMatrix
-
 #endif // define MISC_LINEAR_ALGEBRA_H
 
